import os
import unittest
from modelbest_sdk.file_format.mbtable_builder import MbTableBuilder
from modelbest_sdk.file_format.mbtable_partition import MbTablePartition, MbTablePartitionIterator


class TestMbTablePartition(unittest.TestCase):
    def setUp(self):
        for i in range(10):
            builder = MbTableBuilder(f"test/partition_data/part_{i}.mbt")
            for j in range(100):
                builder.write(str({"col": f"value_{j}"}))
            builder.add_metadata("meta_key", "meta_value")
            builder.flush()
        self.mbtable_partition = MbTablePartition("test/partition_data")
    
    def tearDown(self):
        for i in range(10):
            file_path = f"test/partition_data/part_{i}.mbt"
            if os.path.exists(file_path):
                os.remove(file_path)
                
    def test_get_total_count(self):
        self.mbtable_partition.get_total_count()
        assert self.mbtable_partition.total_count == 1000
                
    def test_get_file_and_position(self):
        index, postion = self.mbtable_partition.get_file_index_and_position(0)
        assert index == 0
        assert postion == 0
        
        index, postion = self.mbtable_partition.get_file_index_and_position(999)
        assert index == 9
        assert postion == 99
        
        index, postion = self.mbtable_partition.get_file_index_and_position(1001)
        assert index == None
        assert postion == None

    
    def test_get_next_file(self):
        next_file_index = self.mbtable_partition.get_next_file_index(0)
        assert next_file_index == 1
        
        next_file_index = self.mbtable_partition.get_next_file_index(9)
        assert next_file_index == None
        
class TestMbTablePartitionIterator(TestMbTablePartition):
    def test_iterator(self):
        cnt = 0
        with MbTablePartitionIterator(self.mbtable_partition, start_index=0, max_iterations=1000) as iterator:
            for i, record in enumerate(iterator):
                assert record == str({"col": f"value_{i%100}"}).encode()          
                cnt += 1
                assert iterator.current_file_index == i // 100
        assert cnt == 1000


if __name__ == '__main__':
    unittest.main()


from modelbest_sdk.file_format.mbtable_builder import MbTableBuilder


for i in range(10):
    builder = MbTableBuilder(f"test/partition_data/part_{i}.mbt")
    for j in range(100):
        builder.write(f"file_{i}_value_{j}")
    builder.add_metadata("meta_key", "meta_value")
    builder.flush()


import json
import os
import unittest
from modelbest_sdk.file_format.mbtable import MbTable, MbTableIterator
from modelbest_sdk.file_format.mbtable_builder import MbTableBuilder


class TestMbTable(unittest.TestCase):
    def setUp(self):
        self.path = "test/data/single_file_padding_key.mbt"
        builder = MbTableBuilder(self.path)
        for i in range(100):
            builder.write(json.dumps({"col": f"value_{i}"}))
        builder.add_metadata("meta_key", "meta_value")
        builder.flush()
        self.mbtable = MbTable(self.path)
    
    def tearDown(self):
        os.remove(self.path)
    
    def test_get_record(self):
        self.assertEqual(self.mbtable.read(0), json.dumps({"col": "value_0"}).encode())
        self.assertEqual(self.mbtable.read(99), json.dumps({"col": "value_99"}).encode())
        self.assertEqual(self.mbtable.read(100), b'')
        
    def test_get_metadata(self):
        self.assertEqual(self.mbtable.get_metadata("meta_key"), b"meta_value")
        self.assertEqual(self.mbtable.get_metadata("not_exist"), b'')
    
    def test_get_entry_count(self):
        self.assertEqual(self.mbtable.get_entry_count(), 100)
    
    
class TestMbTableIterator(TestMbTable):
    def test_iterator(self):
        cnt = 0
        with MbTableIterator(self.path) as iterator:    
            for record in iterator:
                self.assertEqual(record, json.dumps({"col": f"value_{cnt}"}).encode())
                cnt += 1
        assert cnt == 100
    
    def test_iterator_with_args(self):
        key, cnt = 50, 0
        with MbTableIterator(self.path, start_index=key, max_iterations=10) as iterator:  
            for _ in iterator:
                cnt += 1
        assert cnt == 10
        
        key, cnt = 99, 0
        with MbTableIterator(self.path, start_index=key, max_iterations=1000) as iterator:  
            for _ in iterator:
                cnt += 1
        assert cnt == 1
    
        key, cnt = 1000, 0
        # !!! note that key in mbtable is string and is sorted in lexicographic order
        # so key 1000 is greater than 10 but less than 11, this would start from 11
        with MbTableIterator(self.path, start_index=key, max_iterations=1000) as iterator:  
            for _ in iterator:
                cnt += 1
        assert cnt == 0
    
    def test_iterator_raw(self):
        iterator = MbTableIterator(self.path)
        iterator.__enter__()
        record = iterator.__next__()
        iterator.__exit__(None, None, None)
        self.assertEqual(record, json.dumps({"col": "value_0"}).encode())
        pass
        

                
if __name__ == '__main__':
    unittest.main()
    